
#include "DisplayWindow.h"
#include <iostream>
#include <cstdlib>
using namespace std;
#pragma hdrstop

// -------------------------------------------------------- forward declarations

void LINE(string* args);
void FLUSH(string* args);
void CIRCLE(string* args);
void FILLCIRCLE(string* args);
void FINISH(string* args);
void QUIT(string* args);
void WRITE(string* args);
void CLEAR(string* args);
void POLYGON(string* args);
void FILLPOLYGON(string* args);
void SAVE(string* args);
void LOAD(string* args);
void GETPIXEL(string* args);
void SETCOLOR(string* args);
void PUTPIXEL(string* args);
void ARC(string* args);
void RECTANGLE(string* args);
void FILLARC(string* args);

// ---------------------------------------------------------------- main program

bool Quit = false;

DisplayWindow *Win;

int known_command_count = 17;
string commands[] = {"ARC", "CIRCLE", "CLEAR", "FILLARC", "FILLCIRCLE", "FILLPOLYGON", "FINISH", "FLUSH", "GETPIXEL",
                     "LINE", "LOAD", "POLYGON", "PUTPIXEL", "QUIT", "RECTANGLE", "SAVE", "SETCOLOR", "WRITE"};
void ((*functions[])(string* args)) = {ARC, CIRCLE, CLEAR, FILLARC, FILLCIRCLE, FILLPOLYGON, FINISH, FLUSH, GETPIXEL,
                                       LINE, LOAD, POLYGON, PUTPIXEL, QUIT, RECTANGLE, SAVE, SETCOLOR, WRITE};
int argument_counts[] = {5,3,0,6,3,1,0,0,2,4,1,1,2,0,4,1,3,3};

string *arguments = new(string[6]);


int main(int argc, char** argv)
{

    int w, h; // --------------------------------------- command line parameters
    string caption="PipeGraph", iconpath="None";
    if(argc < 3)
    {
        cerr << argv[0] << ": need at least two parameters, width and height of the window\n";
        exit(1);
    }
    else 
    {
        w=atoi(argv[1]);
        h=atoi(argv[2]);
        if(argc > 3)
            caption = argv[3];
        if(argc > 4)
            iconpath = argv[4];
    }
    Win = new(DisplayWindow)(w,h,caption,iconpath);

    
    string command; // --------------------------------- read command from stdin
    while(!Quit)
    {
        cin >> command;
        int CommandIndex = 0;
        for(; CommandIndex < known_command_count; CommandIndex++)
            if(command == commands[CommandIndex])
                break;
            
        if(CommandIndex < known_command_count)
        {
            for(int j = 0; j < argument_counts[CommandIndex]; j++)
                cin >> arguments[j];
            functions[CommandIndex](arguments);
        }
        else
            cerr << "Unknown Command \"" << command << "\"\n";
    }


    delete[](arguments);
    delete(Win);
    return 0;
}


// ------------------------------------------------------------------ primitives



void LINE(string* args)
{
    Win->Line(atoi(args[0].c_str()),atoi(args[1].c_str()),atoi(args[2].c_str()),atoi(args[3].c_str()));
}

void FLUSH(string* args)
{
    Win->Flush();
}

void QUIT(string* args)
{
    Quit = true;
}

void WRITE(string* args)
{
    Win->WriteText(atoi(args[0].c_str()), atoi(args[1].c_str()), args[2]);
}

void CLEAR(string* args)
{
    Win->Clear();
}

void CIRCLE(string* args)
{
    Win->Circle(atoi(args[0].c_str()),atoi(args[1].c_str()),atoi(args[2].c_str()));
}

void FILLCIRCLE(string* args)
{
    Win->FillCircle(atoi(args[0].c_str()),atoi(args[1].c_str()),atoi(args[2].c_str()));
}

void POLYGON(string* args)
{
    int n = atoi(args[0].c_str());
    XPoint *Poly=new(XPoint[n]);
    for(int i = 0; i < n; i++)
    {
        cin >> Poly[i].x;
        cin >> Poly[i].y;
    }
    Win->Polygon(Poly, n);
    delete[](Poly);
}

void FILLPOLYGON(string* args)
{
    int n = atoi(args[0].c_str());
    XPoint *Poly=new(XPoint[n]);
    for(int i = 0; i < n; i++)
    {
        cin >> Poly[i].x;
        cin >> Poly[i].y;
    }
    Win->FillPolygon(Poly, n);
    delete[](Poly);
}

void SAVE(string* args)
{
    Win->SaveBitmap(args[0]);
}

void RECTANGLE(string* args)
{
    Win->Rectangle(atoi(args[0].c_str()),atoi(args[1].c_str()),atoi(args[2].c_str()),atoi(args[3].c_str()));
}

void LOAD(string *args)
{
    Win->LoadPixmap(0,0,args[0]);
}

void GETPIXEL(string* args)
{
    Win->GetPixel(atoi(args[0].c_str()),atoi(args[1].c_str()));
}

void SETCOLOR(string* args)
{
    Win->SetColor(atoi(args[0].c_str()),atoi(args[1].c_str()),atoi(args[2].c_str()));
}

void PUTPIXEL(string* args)
{
    Win->PutPixel(atoi(args[0].c_str()),atoi(args[1].c_str()));
}

void ARC(string* args)
{
    Win->Arc(atoi(args[0].c_str()),atoi(args[1].c_str()),atoi(args[2].c_str()),
             atoi(args[3].c_str()),atof(args[4].c_str()));
}

void FILLARC(string* args)
{
    Win->FillArc(atoi(args[0].c_str()),atoi(args[1].c_str()),atoi(args[2].c_str()),
                 atoi(args[3].c_str()),atoi(args[4].c_str()),atoi(args[5].c_str()));
}

void FINISH(string* args)
{
    Win->WaitUntilClosed();
    Quit = true;
}
